<?php

namespace JFalke\Features\SearchBrands;

use Http\Discovery\Psr17FactoryDiscovery;
use JFalke\Exceptions\CouldNotGetBrandsException;
use JFalke\Models\Brand;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\ResponseInterface;
use stdClass;

class SearchBrandsControllerTest extends TestCase
{
    private SearchBrandsService $service;


    public function setUp():void
    {
        $this->service = $this->createMock(SearchBrandsService::class);
    }

    public function testShouldReturn400WhenMissingInputParametersException(): void
    {
        $response = $this->invokeWith(__DIR__ . '/payloads/missing-property.json');
        // La factory de request doit throw une MissingInputParametersException catchée dans le controleur
        $this->assertSame(400, $response->getStatusCode());
    }

    public function testShouldReturn404WhenCouldNotGetBrandsException(): void
    {
        $this->service->expects($this->once())->method('handle')->willThrowException(new CouldNotGetBrandsException());
        $response = $this->invokeWith(__DIR__ . '/payloads/unvalid-payload.json');
        $this->assertSame(404, $response->getStatusCode());
    }

    public function testShouldReturn200WhenBrandSearched():void
    {
        $this->service->expects($this->once())->method('handle')->with(
            new SearchBrandsRequest("ABARTH")
        )->willReturn(new SearchBrandsResponse(new Brand(3854, "ABARTH")));

        $response = $this->invokeWith(__DIR__ . '/payloads/valid-payload.json');
        var_dump($response->getBody()->getContents());
        $this->assertSame(200, $response->getStatusCode());
    }

    private function invokeWith(?string $filename = null): ResponseInterface
    {
        $controller = new SearchBrandsController($this->service, $this->twig);

        $request = Psr17FactoryDiscovery::findServerRequestFactory()
            ->createServerRequest('POST', 'random_uri');

        if ($filename !== null && is_file($filename)) {
            $request = $request->withBody(
                Psr17FactoryDiscovery::findStreamFactory()->createStream(
                    file_get_contents($filename)
                )
            );
        }

        return $controller->__invoke(
            $request,
            Psr17FactoryDiscovery::findResponseFactory()
                ->createResponse()
        );
    }
}