<?php

namespace JFalke\Features\SearchBrands;

use JFalke\Exceptions\CouldNotGetBrandsException;
use JFalke\Features\SearchBrands\SearchBrandsRequest;
use JFalke\Features\SearchBrands\SearchBrandsResponse;
use JFalke\Features\SearchBrands\SearchBrandsService;
use JFalke\Repositories\BrandRepository;
use PHPUnit\Framework\TestCase;
use stdClass;

class SearchBrandsServiceTest extends TestCase
{

    public function testShouldThrowCouldNotGetBrandsException(){
        $this->expectException(CouldNotGetBrandsException::class);

        $request = new SearchBrandsRequest("zzzzzzz");

        $brandRepository = $this->createMock(BrandRepository::class);

        $brandRepository->expects($this->once())->method('getBrands')
            ->with('zzzzzzz')
            ->willReturn(null);

        $service = new SearchBrandsService($brandRepository);

        $service->handle($request);
    }

    public function testShouldReturnResponseBrandNameToutes(){
        $request = new SearchBrandsRequest('toutes');

        $brandRepository = $this->createMock(BrandRepository::class);
        $brandRepository->expects($this->once())->method('isToutes')
            ->with('toutes')
            ->willReturn(true);

        $brandRepository->expects($this->once())->method('getBrands')
            ->with('toutes')
            ->willReturn(array());

        $service = new SearchBrandsService($brandRepository);

        $response = $service->handle($request);
        $this->assertInstanceOf(SearchBrandsResponse::class, $response);
        $this->assertSame(array(), $response->getBrand());
    }
}